import { Context } from '../context';
import { ContractStorageType, DefaultContractType } from '../contract/contract';
import { ContractMethod } from '../contract/contract-methods/contract-method-flat-param';
import { SendParams } from '../contract/contract-methods/contract-method-interface';
import { ContractProvider } from '../contract/interface';
import { BatchOperation } from '../operations/batch-operation';
import { OperationEmitter } from '../operations/operation-emitter';
import { ActivationParams, DelegateParams, OriginateParams, TransferParams, ParamsWithKind, RegisterGlobalConstantParams, TxRollupOriginateParams, TxRollupBatchParams, TransferTicketParams, IncreasePaidStorageParams } from '../operations/types';
import { OpKind } from '@taquito/rpc';
import { ContractMethodObject } from '../contract/contract-methods/contract-method-object-param';
import { EstimationProvider } from '../estimate/estimate-provider-interface';
export declare const BATCH_KINDS: OpKind[];
export declare type BatchKinds = OpKind.ACTIVATION | OpKind.ORIGINATION | OpKind.TRANSACTION | OpKind.DELEGATION;
export declare class OperationBatch extends OperationEmitter {
    private estimator;
    private operations;
    constructor(context: Context, estimator: EstimationProvider);
    /**
     *
     * @description Add a transaction operation to the batch
     *
     * @param params Transfer operation parameter
     */
    withTransfer(params: TransferParams): this;
    /**
     *
     * @description Transfer tickets from a Tezos address (tz1,tz2 or tz3) to a smart contract address( KT1)
     *
     * @param params Transfer operation parameter
     */
    withTransferTicket(params: TransferTicketParams): this;
    /**
     *
     * @description Add a contract call to the batch
     *
     * @param params Call a contract method
     * @param options Generic operation parameters
     */
    withContractCall(params: ContractMethod<ContractProvider> | ContractMethodObject<ContractProvider>, options?: Partial<SendParams>): this;
    /**
     *
     * @description Add a delegation operation to the batch
     *
     * @param params Delegation operation parameter
     */
    withDelegation(params: DelegateParams): this;
    /**
     *
     * @description Add an activation operation to the batch
     *
     * @param params Activation operation parameter
     */
    withActivation({ pkh, secret }: ActivationParams): this;
    /**
     *
     * @description Add an origination operation to the batch
     *
     * @param params Origination operation parameter
     */
    withOrigination<TContract extends DefaultContractType = DefaultContractType>(params: OriginateParams<ContractStorageType<TContract>>): this;
    /**
     *
     * @description Add an operation to register a global constant to the batch
     *
     * @param params RegisterGlobalConstant operation parameter
     */
    withRegisterGlobalConstant(params: RegisterGlobalConstantParams): this;
    /**
     *
     * @description Add an operation to increase paid storage to the batch
     *
     * @param params IncreasePaidStorage operation parameter
     */
    withIncreasePaidStorage(params: IncreasePaidStorageParams): this;
    /**
     *
     * @description Add an operation to originate a rollup to the batch
     *
     * @param params Rollup origination operation parameter
     */
    withTxRollupOrigination(params?: TxRollupOriginateParams): this;
    /**
     *
     * @description Add an operation to submit a tx rollup batch to the batch
     *
     * @param params Tx rollup batch operation parameter
     */
    withTxRollupSubmitBatch(params: TxRollupBatchParams): this;
    private getRPCOp;
    /**
     *
     * @description Add a group operation to the batch. Operation will be applied in the order they are in the params array
     *
     * @param params Operations parameter
     */
    with(params: ParamsWithKind[]): this;
    /**
     *
     * @description Forge and Inject the operation batch
     *
     * @param params Optionally specify the source of the operation
     */
    send(params?: {
        source?: string;
    }): Promise<BatchOperation>;
}
export declare class RPCBatchProvider {
    private context;
    private estimator;
    constructor(context: Context, estimator: EstimationProvider);
    /***
     *
     * @description Batch a group of operation together. Operations will be applied in the order in which they are added to the batch
     *
     * @param params List of operation to batch together
     */
    batch(params?: ParamsWithKind[]): OperationBatch;
}
